var fs = require('fs');

module.exports = class DropDB
{
  /**
  * コンストラクタ
  * 指定したファイルパスが存在すれば、読み込んでデータモデルとして使用する。
  * 存在しなければ、空のデータモデルを作成します。
  */
  /*
  constructor (filepath)
  {
    this.filepath = filepath;
    if(this.isExistFile(filepath))
    {
      this.model = JSON.parse(fs.readFileSync(this.filepath,'UTF-8'));
    }
    else
    {
      this.model = new Array();
    }
  }
  */

  constructor(dirpath, filepath)
  {
    this.filepath = filepath
    this.dirpath = dirpath
    // ディレクトリがなければ作成する。
    if(!fs.existsSync(dirpath))
    {
      fs.mkdirSync(dirpath)
    }

    if(this.isExistFile(dirpath + '/' + filepath))
    {
      this.model = JSON.parse(fs.readFileSync(this.dirpath + '/' + this.filepath,'UTF-8'));
    }
    else
    {
      this.model = new Array();
    }
  }

  /**
  * データモデル表示関数
  * コンソールにデータモデルを表示させます。
  */
  show()
  {
    console.log(this.dirpath);
    console.log(this.filepath)
    console.log(this.model);
  }

  /**
  * ファイル存在確認関数
  * 指定したファイルパスが正しいかどうかを返します。
  */
  isExistFile(filepath)
  {
    try {
      fs.statSync(filepath);
      return true;
    } catch (e) {
      if(e.code === 'ENOENT') return false;
    } finally {

    }
  }

  /**
  * 検索関数
  * 指定したクエリと一致したデータモデルの配列を返します。
  */
  find(query)
  {
    var findNum = this.search(query);
    var result = new Array();

    for( var i = 0; i < findNum.length; i++ )
    {
      result.push( this.model[findNum[i]] );
    }

    return result;
  }

  /**
  * 全検索関数
  * データモデルをすべて返します。
  */
  findAll()
  {
    return this.model;
  }

  /**
  * 挿入関数
  * 指定したクエリのデータをデータモデルに追加します。
  */
  insert(query)
  {
    this.model.push(query);
    this.writeFileSync();
  }

  /**
  * 挿入関数
  * 指定したクエリのデータをデータモデルに追加します。
  */
 insert2(query)
 {
   this.model.push(query);
   //this.writeFileSync();
 }

  /**
  * 削除関数
  * 指定したクエリと一致したデータモデルを削除します。
  */
  delete(query)
  {
    var delNum = this.search(query);

    //配列を削除する度に対象番号がーｎされる
    for( var i = 0; i < delNum.length; i++ )
    {
      this.model.splice( delNum[i]-i , 1 );
    }
    this.writeFileSync();
  }

  /**
  * 削除関数
  * 指定したクエリと一致したデータモデルを削除します。
  */
 delete2(query)
 {
   var delNum = this.search(query);

   //配列を削除する度に対象番号がーｎされる
   for( var i = 0; i < delNum.length; i++ )
   {
     this.model.splice( delNum[i]-i , 1 );
   }
   //this.writeFileSync();
  }

  /**
  * 更新関数
  * 指定したsearchと一致したデータモデルを指定したクエリのデータに更新します。
  */
  update(search,query)
  {
    var updNum = this.search(search);

    for( var q_key in query )
    {
      for( var i = 0; i < updNum.length; i++ )
      {
        this.model[updNum[i]][q_key] = query[q_key];
      }
    }

    this.writeFileSync();
  }

  /**
  * 更新関数
  * 指定したsearchと一致したデータモデルを指定したクエリのデータに更新します。
  */
 update2(search,query)
 {
   var updNum = this.search(search);

   for( var q_key in query )
   {
     for( var i = 0; i < updNum.length; i++ )
     {
       this.model[updNum[i]][q_key] = query[q_key];
     }
   }

   //this.writeFileSync();
 }

  /**
  * 計数関数
  * データモデルの要素数を返します。
  */
  count()
  {
    return this.model.length;
  }

  /**
  * 指定したクエリと一致したデータモデルの番号を返します。
  */
  search(query)
  {
    var tmpNum = new Array();
    for( var q_key in query )
    {
      this.model.some( function( element , index ) {
        if( query[q_key] === element[q_key] ) tmpNum.push( index );
      } );
    }
    return tmpNum;
  }

  /**
  * 保存関数
  * データモデルの状態をファイルに保存します。
  */
  writeFileSync()
  {
    fs.writeFileSync(this.dirpath + '/' + this.filepath,JSON.stringify(this.model,null,"\t"));
  }
}
